import sys
import os
import json
import re
import urllib
from urllib.request import urlopen
from urllib.parse import urlparse

# URL to locally hosted Nueval rules JSON
ASSESSMENT_BASE_URL = "http://fabricademy-machinable-rules.test/json/"

ASSESSMENT_DOCS_DIR = os.path.join(os.getcwd(), 'docs/assessment')


def make_task(index, task, unit_data):
    """ Create a Markdown fragment for a task """
    task_md = "\n"
    task_md += '## %s. %s\n' % (index, task['name'])
    # task_md += '> :sparkles: Updated for %s evaluation standards!\n' % unit_data['version']
    if task['description']:
        task_md += task['description']
    task_md += '\n**Learning outcomes**\n'
    for task_outcome in task['outcomes']:
        task_md += '\n- %s' % task_outcome
    task_md += '\n\n**Student checklist**\n'
    for task_checklist in task['checklist']:
        task_md += '\n- [x] %s' % task_checklist
    return task_md

pattern = r"### (.*?)\n>(.*?)(?=\n###|\Z)"
def replace_match(match):
    question = match.group(1).strip()
    answer = match.group(2).strip().replace('\n', '\n\n')      
    return '!!! question "{}"\n    {}'.format(question, answer)

def make_markdown_doc(index, unit_data):
    """ Create a Markdown version of a unit """
    md_buffer = ""
    # md_buffer += '### %s\n' % unit_data['unit']
    for task in unit_data['tasks']:
        md_buffer += make_task(index, task, unit_data)
    if "None yet" not in unit_data['faq']:
        md_buffer += '\n\n**FAQ**\n'
        # res = re.finditer(r"Q[:\.] (.*)\nA[:\.] (.*)",  unit_data['faq'])
        # for match in res:
        #     md_buffer += "\n!!! question \"" + match.group(1) + "\""
        #     md_buffer += "\n    " + match.group(2)
        
        new_text = re.sub(pattern, replace_match, unit_data['faq'], flags=re.DOTALL)
        md_buffer += "\n"
        md_buffer += new_text
    md_buffer += '\n\n---'
    # issueLinkTitle = urlparse('Assessment question: %s' % unit_data['unit'])
    # issueLinkDesc = urlparse('Write your question here...')
    # md_buffer += ':wink: Have a question or feedback? Click [this link](https://gitlab.fabcloud.org/{0}/{1}/-/issues/new?issue[title]={2}&issue[description]={3}) to open an issue!'.format(ACADEMY_GITLAB_BASEPATH, unit_data['version'], issueLinkTitle, issueLinkDesc)
    return md_buffer


content = urlopen(ASSESSMENT_BASE_URL + "rules.json")
jsonList = json.loads(content.read())

markdownContent = "---\nfyi: AUTO Generated file!\n---\n"
markdownContent += "# Assessment Criteria\n"
markdownContent += "\n> Classes: September - December\n"
markdownContent += """
## 0. Recitations and Class attendance

**Learning outcomes**

- Become part of the Fabricademy community and share your ideas, thoughts, opportunities and projects

**Student checklist**

- [x] Participate in the weekly recitations
- [x] Participate in the weekly Lectures
- [x] Participate in the weekly reviews
- [x] Participate in the weekly tutorials

---
"""

# yaml_file = open('firstblock.json', 'r')
#     yaml_data = yaml.load(yaml_file.read()
# markdownContent += make_markdown_doc(index, ruleSetList) + "\n\n"

index = 1
for fileName in jsonList:
    print("Fetch data from " + fileName)
    ruleSetContent = urlopen(ASSESSMENT_BASE_URL + fileName)
    ruleSetList = json.loads(ruleSetContent.read())
    markdownContent += make_markdown_doc(index, ruleSetList) + "\n\n"
    index += 1

tmp_file = open(os.path.join(ASSESSMENT_DOCS_DIR, 'criteria.md'), 'w', encoding='utf-8')
tmp_file.write(markdownContent)
tmp_file.close()

print("Succes !!")

# sys.exit(-1)
